/* -*- Mode: C++ -*-
 * Worldvisions Weaver Software:
 *   Copyright (C) 1997-2002 Net Integration Technologies, Inc.
 *
 * Declares WvIPNetList
 *
 */

#ifndef __WVIPNETLIST_H
#define __WVIPNETLIST_H

#include "wvlinklist.h"
#include "wvaddr.h"

DeclareWvList(WvIPNet);

#endif // __WVIPNETLIST_H
