/*
 * Worldvisions Weaver Software:
 *   Copyright (C) 1997-2002 Net Integration Technologies, Inc.
 * 
 * A UniConf moniker that uses an .ini file to look up which moniker it
 * should use to find the config file/subtree for a particular application.
 */
#ifndef __UNIAUTOGEN_H
#define __UNIAUTOGEN_H

class WvString;

extern WvString uniautogen_moniker;

#endif // __UNIAUTOGEN_H
